	SUBROUTINE SFORCES
************************************************************
*   Calculates derivative of the total energy over nuclear *
*   displacements. The Pulay forces are assumed to be      *
*   taken in "rigid" approximation.                        *
*   Comments:                                              *
*   A few formulas are used to calculate the forces.       * 
*   When the charge density is self-consistent and         *
*   the number of plane waves is sufficient, all of        *
*   them must coinside. On the other hand, the derivative  *
*   of the energy functional programmed in this code is    *
*   most likely given by the OKA1 force formula.           *
*   The OKA2 force formula is the same, but if the one-    *
*   center expansions for the LMTOs and the Coulomb        *
*   potential are convergent. The OKA3 force is equal      *
*   to OKA2 force when the self-consistency is reached.    *
*   The Hellmann-Feynman force plus Pulay and core correc- *
*   tions are very sensitive to the self-consistency and   *
*   the # of plane waves used. Compare different surface   *
*   contributions to understand whether accuracy is        *
*   sufficient.                                            *
*   More comments:                                         *
*   If NPF=0, only Hellman-Feynman forces plus core        *
*   correction are available. This is not accurate         *
*   force formula for using with the LMTO method!          *
*   If NPF=1, full calculation of forces using             *
*   different rearrangements in the force formula          *
*   are available, but this might be too slow for          *
*   many-atom systems.                                     *
*   If NPF=2, only OKA1 and HF - formulas as well as the   *
*   comparison between surface integrals are available.    *
*   This is 6 times faster than NPF=1 calculation.         *
*   Written by S.Yu.Savrasov in Max-Planck Institute       *
*   fur Festkoerperforschung, Stuttgart, Germany.          *
*   Created 23.03.95, Last edited 25.04.95.                *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX),IS(NPAT)
	COMPLEX*16 FPTN(3,NPAT,4),FCOU(3,NPAT,4),FEXC(3,NPAT,4)
     &    ,FBND(3,NPAT,4),FMLT(3,NPAT,4),FMT0(3,NPAT,4),FTOT(3,NPAT,4)
     &    ,FHLF(3,NPAT,4),SUMRUL(3,4),RESRHO(NPAT,NPANMAX,NSPINMAX)
     &    ,PFVAL(-1:1,NPAT,NPANMAX,NSPINMAX,2),PFM(-1:1,2),PFX(3,2)
	DIMENSION  TAU(3,NPAT)
	CHARACTER*10 TXTEL(NSORTMAX)
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM       !!! from init
	COMMON /FORCES/ FPTN,FCOU,FEXC,FHLF                 !!! from forces
	COMMON /PILAUF/ PFVAL,RESRHO                        !!! from rhodot
	COMMON /BASIS/  TAU                                 !!! from init
        COMMON /KAPPAR/ NPAN                                !!! from init
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	DATA IPRINT/1/               !!! full output
C   CALCULATE DIFFERENT FORCES, SOME OF THEM ARE COMPUTED BY "ENERGY" PROGRAM
	CALL BNDFORCE(FBND)          !!! band contribution to the force
	CALL MLTFORCE(FMLT)          !!! multipole contribution to the force
	CALL MT0FORCE(FMT0)          !!! mt-zero contribution to the force
C *** SET DIFFERENT CONTRIBUTIONS TO FORCES ***
	DO 20 IATOM=1,NATOM
	ISORT=IS(IATOM)
	DO 10 I=1,3
C   THE FIRST FORMULA FOR THE OKA FORCE
	FTOT(I,IATOM,1)=FBND(I,IATOM,1)+FBND(I,IATOM,2)-
     &  0.5D0*FCOU(I,IATOM,1)-0.5D0*FCOU(I,IATOM,2)+FMLT(I,IATOM,1)-
     &  FEXC(I,IATOM,1)-FEXC(I,IATOM,2)+FMT0(I,IATOM,1)
C   THE SECOND FORMULA FOR THE OKA FORCE
	FTOT(I,IATOM,2)=FBND(I,IATOM,1)+FBND(I,IATOM,2)-
     &  FCOU(I,IATOM,1)-FEXC(I,IATOM,1)-FEXC(I,IATOM,2)+
     &  FMT0(I,IATOM,1)
C   THE THIRD FORMULA FOR THE OKA FORCE
	FTOT(I,IATOM,3)=FBND(I,IATOM,1)+FBND(I,IATOM,2)-
     &  FPTN(I,IATOM,1)-FCOU(I,IATOM,3)
C   THE HELLMANN-FEYNMAN + PULAY FORCE FORMULA
	IF(NPF.EQ.1)
     &	FTOT(I,IATOM,4)=FBND(I,IATOM,1)+FBND(I,IATOM,2)-
     &  FPTN(I,IATOM,4)+FHLF(I,IATOM,2)+FHLF(I,IATOM,1)
C   THE HELLMANN-FEYNMAN + CORE CORRECTION (NO PULAY IS AVAIABLE)
	IF(NPF.EQ.0.OR.NPF.EQ.2)
     &  FTOT(I,IATOM,4)=-FPTN(I,IATOM,3)+FHLF(I,IATOM,3)+
     &  FHLF(I,IATOM,1)
C   SET ZEROES
	DO 10 IC=1,4
	IF(ABS(FBND(I,IATOM,IC)).LE.1.D-10)FBND(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FMLT(I,IATOM,IC)).LE.1.D-10)FMLT(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FMT0(I,IATOM,IC)).LE.1.D-10)FMT0(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FPTN(I,IATOM,IC)).LE.1.D-10)FPTN(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FCOU(I,IATOM,IC)).LE.1.D-10)FCOU(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FEXC(I,IATOM,IC)).LE.1.D-10)FEXC(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FHLF(I,IATOM,IC)).LE.1.D-10)FHLF(I,IATOM,IC)=(0.D0,0.D0)
	IF(ABS(FTOT(I,IATOM,IC)).LE.1.D-10)FTOT(I,IATOM,IC)=(0.D0,0.D0)
  10    CONTINUE
  20	CONTINUE
C   CHECK OUT FOR SUM RULE
	DO 40 I=1,3
	DO 40 IC=1,4
	SUMRUL(I,IC)=(0.D0,0.D0)
	DO 30 IATOM=1,NATOM
	ISORT=IS(IATOM)
	SUMRUL(I,IC)=SUMRUL(I,IC)+FTOT(I,IATOM,IC)
  30	CONTINUE
  40	CONTINUE
C *** PRINT BLOCK ***
	WRITE(IUN,1000)
C  PRINT TOTAL FORCES: OKA1,OKA2,OKA3,HF+PULAY OR HF+CORE CORRECTION
	DO 110 IATOM=1,NATOM
	ISORT=IS(IATOM)
	WRITE(IUN,1010)(TAU(I,IATOM),I=1,3),TXTEL(ISORT)
	IF(NPF.EQ.1.OR.NPF.EQ.2)
     &	WRITE(IUN,1020)(-DREAL(FTOT(I,IATOM,1)),I=1,3)
	IF(NPF.EQ.1)
     &	WRITE(IUN,1030)(-DREAL(FTOT(I,IATOM,2)),I=1,3)
	IF(NPF.EQ.1)
     &	WRITE(IUN,1040)(-DREAL(FTOT(I,IATOM,3)),I=1,3)
	IF(NPF.EQ.1)
     &	WRITE(IUN,1050)(-DREAL(FTOT(I,IATOM,4)),I=1,3)
	IF(NPF.EQ.0.OR.NPF.EQ.2)
     &	WRITE(IUN,1061)(DREAL(-FHLF(I,IATOM,1)),I=1,3)
	IF(NPF.EQ.0.OR.NPF.EQ.2)
     &	WRITE(IUN,1062)
     &  (DREAL(+FPTN(I,IATOM,3)-FHLF(I,IATOM,3)),I=1,3)
	IF(NPF.EQ.0.OR.NPF.EQ.2)
     &	WRITE(IUN,1063)(DREAL(-FTOT(I,IATOM,4)),I=1,3)
 110	CONTINUE
1000   FORMAT(1X,'* CALCULATED FORCES AT THE CENTERS OF ATOMS ------>')
1010   FORMAT(1X,'* Position --> (',G12.5,',',G12.5,',',G12.5,') ',A10)
1020   FORMAT(1X,'* OKA 1 FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1030   FORMAT(1X,'* OKA 2 FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1040   FORMAT(1X,'* OKA 3 FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1050   FORMAT(1X,'* HF+PL FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1061   FORMAT(1X,'* HF    FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1062   FORMAT(1X,'* CR    FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1063   FORMAT(1X,'* HF+CR FORCE: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
C  PRINT SUM RULES
	WRITE(IUN,1070)
	IF(NPF.EQ.1.OR.NPF.EQ.2)
     &	WRITE(IUN,1071)(DREAL(-SUMRUL(I,1)),I=1,3)
	IF(NPF.EQ.1)
     &	WRITE(IUN,1072)(DREAL(-SUMRUL(I,2)),I=1,3)
	IF(NPF.EQ.1)
     &	WRITE(IUN,1073)(DREAL(-SUMRUL(I,3)),I=1,3)
	IF(NPF.EQ.1)
     & 	WRITE(IUN,1074)(DREAL(-SUMRUL(I,4)),I=1,3)
	IF(NPF.EQ.0.OR.NPF.EQ.2)
     &	WRITE(IUN,1075)(DREAL(-SUMRUL(I,4)),I=1,3)
1070   FORMAT(1X,'* CALCULATED SUM RULES FOR THESE FORCES     ------>')
1071   FORMAT(1X,'* OKA 1 SUMRL: Sx=',G14.7,'; Sy=',G14.7,'; Sz=',G14.7)
1072   FORMAT(1X,'* OKA 2 SUMRL: Sx=',G14.7,'; Sy=',G14.7,'; Sz=',G14.7)
1073   FORMAT(1X,'* OKA 3 SUMRL: Sx=',G14.7,'; Sy=',G14.7,'; Sz=',G14.7)
1074   FORMAT(1X,'* HF+PL SUMRL: Sx=',G14.7,'; Sy=',G14.7,'; Sz=',G14.7)
1075   FORMAT(1X,'* HF+CR SUMRL: Sx=',G14.7,'; Sy=',G14.7,'; Sz=',G14.7)
C  PRINT DIFFERENT CONTRIBUTIONS IF NECESSARY
	IF(NPF.EQ.0)RETURN
	IF(IPRINT.NE.1)RETURN
	WRITE(IUN,1091)
	WRITE(IUN,1092)
	DO 310 IATOM=1,NATOM     !!! over displacements
	ISORT=IS(IATOM)
	WRITE(IUN,1093)(TAU(I,IATOM),I=1,3),TXTEL(ISORT)
1091   FORMAT(1X,'***************************************************')
1092   FORMAT(1X,'* DIFFERENT CONTRIBUTIONS TO DERIVATIVES    ------>')
1093   FORMAT(1X,'* Position --> (',G12.5,',',G12.5,',',G12.5,') ',A10)
C THE FIRST OKA FORCE CONTRIBUTIONS
	IF(NPF.EQ.1.OR.NPF.EQ.2)THEN
	WRITE(IUN,1100)
	WRITE(IUN,1110)
     &  (DREAL(FBND(I,IATOM,1)+FBND(I,IATOM,2)),I=1,3)
	WRITE(IUN,1120)(DREAL(FMLT(I,IATOM,1)),I=1,3)
	WRITE(IUN,1130)(DREAL(-0.5D0*FCOU(I,IATOM,1)-
     &  0.5D0*FCOU(I,IATOM,2)+FMLT(I,IATOM,1)),I=1,3)
	WRITE(IUN,1140)
     &  (DREAL(-FEXC(I,IATOM,1)-FEXC(I,IATOM,2)),I=1,3)
	WRITE(IUN,1150)(DREAL(FMT0(I,IATOM,1)),I=1,3)
	WRITE(IUN,1160)(DREAL(FTOT(I,IATOM,1)),I=1,3)
	ENDIF
1100   FORMAT(1X,'* OKA 1 Contributions to the Derivatives    ------>')
1110   FORMAT(1X,'* bnd   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1120   FORMAT(1X,'* mlt   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1130   FORMAT(1X,'* cou 1 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1140   FORMAT(1X,'* exc   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1150   FORMAT(1X,'* mtz   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1160   FORMAT(1X,'* tot 1 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
C THE SECOND OKA FORCE CONTRIBUTIONS
	IF(NPF.EQ.1)THEN
	WRITE(IUN,1200)
	WRITE(IUN,1210)
     &  (DREAL(FBND(I,IATOM,1)+FBND(I,IATOM,2)),I=1,3)
	WRITE(IUN,1220)(DREAL(-FCOU(I,IATOM,1)),I=1,3)
	WRITE(IUN,1230)
     &  (DREAL(-FEXC(I,IATOM,1)-FEXC(I,IATOM,2)),I=1,3)
	WRITE(IUN,1240)(DREAL(FMT0(I,IATOM,1)),I=1,3)
	WRITE(IUN,1250)(DREAL(FTOT(I,IATOM,2)),I=1,3)
	ENDIF
1200   FORMAT(1X,'* OKA 2 Contributions to the Derivatives    ------>')
1210   FORMAT(1X,'* bnd   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1220   FORMAT(1X,'* cou 2 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1230   FORMAT(1X,'* exc   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1240   FORMAT(1X,'* mtz   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1250   FORMAT(1X,'* tot 2 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
C THE THIRD OKA FORCE CONTRIBUTIONS
	IF(NPF.EQ.1)THEN
	WRITE(IUN,1300)
	WRITE(IUN,1310)
     &  (DREAL(FBND(I,IATOM,1)+FBND(I,IATOM,2)),I=1,3)
	WRITE(IUN,1320)(DREAL(-FPTN(I,IATOM,1)),I=1,3)
	WRITE(IUN,1330)(DREAL(-FCOU(I,IATOM,3)),I=1,3)
	WRITE(IUN,1340)(DREAL(FTOT(I,IATOM,3)),I=1,3)
1300   FORMAT(1X,'* OKA 3 Contributions to the Derivatives    ------>')
1310   FORMAT(1X,'* bnd   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1320   FORMAT(1X,'* ptn   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1330   FORMAT(1X,'* cou 3 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1340   FORMAT(1X,'* tot 3 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
	ENDIF
C THE HELLMANN-FEYNMAN + PULAY FORCE CONTRIBUTIONS
	IF(NPF.EQ.1)THEN
	WRITE(IUN,1400)
	WRITE(IUN,1410)
     &  (DREAL(FBND(I,IATOM,1)+FBND(I,IATOM,2)-FPTN(I,IATOM,4)+
     &  FPTN(I,IATOM,3)+FHLF(I,IATOM,2)-FHLF(I,IATOM,3)),I=1,3)
	WRITE(IUN,1420)
     &  (DREAL(-FPTN(I,IATOM,3)+FHLF(I,IATOM,3)),I=1,3)
	WRITE(IUN,1430)(DREAL(FHLF(I,IATOM,1)),I=1,3)
	WRITE(IUN,1440)
     &  (DREAL(FPTN(I,IATOM,4)-FHLF(I,IATOM,2)-FPTN(I,IATOM,1)-
     &  FCOU(I,IATOM,3)),I=1,3)
	WRITE(IUN,1450)(DREAL(FTOT(I,IATOM,4)),I=1,3)
	ENDIF
1400   FORMAT(1X,'* Hellmann-Feynman and Pulay Derivatives    ------>')
1410   FORMAT(1X,'* pul   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1420   FORMAT(1X,'* cor   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1430   FORMAT(1X,'* hlf   deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1440   FORMAT(1X,'* hlf " deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1450   FORMAT(1X,'* tot 4 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
C CHECK OUT SURFACE INTERGALS
	IF(NPF.EQ.1.OR.NPF.EQ.2)THEN
	WRITE(IUN,1500)
	WRITE(IUN,1510)(DREAL(FMT0(I,IATOM,1)),I=1,3)
	WRITE(IUN,1520)(DREAL(FMT0(I,IATOM,4)),I=1,3)
	WRITE(IUN,1530)
     &  (DREAL(FPTN(I,IATOM,1)+FPTN(I,IATOM,2)),I=1,3)
	WRITE(IUN,1540)(DREAL(FPTN(I,IATOM,4)),I=1,3)
	WRITE(IUN,1550)
     &  (DREAL(FCOU(I,IATOM,1)+FCOU(I,IATOM,2)),I=1,3)
	WRITE(IUN,1560)(DREAL(FCOU(I,IATOM,4)),I=1,3)
	WRITE(IUN,1570)
     &  (DREAL(FEXC(I,IATOM,1)+FEXC(I,IATOM,2)),I=1,3)
	WRITE(IUN,1580)(DREAL(FEXC(I,IATOM,4)),I=1,3)
	ENDIF
1500   FORMAT(1X,'* Comparison between Surface Integrals      ------>')
1510   FORMAT(1X,'* mt0"p"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1520   FORMAT(1X,'* mt0"s"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1530   FORMAT(1X,'* ptn"p"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1540   FORMAT(1X,'* ptn"s"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1550   FORMAT(1X,'* cou"p"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1560   FORMAT(1X,'* cou"s"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1570   FORMAT(1X,'* exc"p"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1580   FORMAT(1X,'* exc"s"deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
	IF(NPF.EQ.1.OR.NPF.EQ.2)THEN
	WRITE(IUN,1600)
	DO 230 IPAN=1,NPAN
	WRITE(IUN,1610)IPAN
	DO 220 MY=-1,1
	PFM(MY,1)=(0.D0,0.D0)
	PFM(MY,2)=(0.D0,0.D0)
	DO 210 ISPIN=1,NSPIN
	PFM(MY,1)=PFM(MY,1)+PFVAL(MY,IATOM,IPAN,ISPIN,1)
	PFM(MY,2)=PFM(MY,2)+PFVAL(MY,IATOM,IPAN,ISPIN,2)
 210	CONTINUE
 220	CONTINUE
	CALL TRANSFER(PFM(-1,1),PFX(1,1))
	CALL TRANSFER(PFM(-1,2),PFX(1,2))
	WRITE(IUN,1620)(DREAL(PFX(I,1)),I=1,3)
	WRITE(IUN,1630)(DREAL(PFX(I,2)),I=1,3)
	WRITE(IUN,1640)DREAL(RESRHO(IATOM,IPAN,1)+
     &  RESRHO(IATOM,IPAN,NSPIN))/2.D0*DBLE(NSPIN)
 230	CONTINUE
	ENDIF
1600   FORMAT(1X,'* Panel Contributions and Error  Estimates  ------>')
1610   FORMAT(1X,'* Panel # ',I2,' : MT-part,Int-part, Rho-dot error')
1620   FORMAT(1X,'* bnd 1 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1630   FORMAT(1X,'* bnd 2 deriv: Fx=',G14.7,'; Fy=',G14.7,'; Fz=',G14.7)
1640   FORMAT(1X,'* Int {dRho/dE*Vnmt}dr =',G14.7)
 310    CONTINUE               !!! over displacements
	WRITE(IUN,1700)
1700   FORMAT(1X,'***************************************************')
	END




	SUBROUTINE BNDFORCE(FBND)
**************************************************************
*   Calculate band contribution to the Pulay force assuming  *
*   change in the strucure constants only.                   *
*   Written by S.Yu.Savrasov.                                *
**************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER  ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX),IS(NPAT)
	COMPLEX*16 FBND(3,NPAT,4),FSUM(-1:1,2)
     &    ,PFVAL(-1:1,NPAT,NPANMAX,NSPINMAX,2)
	DIMENSION  TAU(3,NPAT)
	CHARACTER  TXTEL(NSORTMAX)*10
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM       !!! from init
	COMMON /BASIS/  TAU                                 !!! from init
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /KAPPAR/ NPAN                                !!! from init
	COMMON /PILAUF/ PFVAL                               !!! from ttrint2
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	DO 10 IATOM=1,NATOM
	DO 10 IXYZ=1,3
	DO 10 IC=1,4
	FBND(IXYZ,IATOM,IC)=(0.D0,0.D0)
  10	CONTINUE
	IF(NPF.EQ.0)RETURN
C  TRANSFERE TO CARTESIAN SYSTEM
	DO 40 IATOM=1,NATOM
	DO 30 MY=-1,1
	FSUM(MY,1)=(0.D0,0.D0)
	FSUM(MY,2)=(0.D0,0.D0)
	DO 20 ISPIN=1,NSPIN
	DO 20 IPAN=1,NPAN
	FSUM(MY,1)=FSUM(MY,1)+PFVAL(MY,IATOM,IPAN,NSPIN,1)
	FSUM(MY,2)=FSUM(MY,2)+PFVAL(MY,IATOM,IPAN,NSPIN,2)
  20    CONTINUE
  30    CONTINUE
	CALL TRANSFER(FSUM(-1,1),FBND(1,IATOM,1))
	CALL TRANSFER(FSUM(-1,2),FBND(1,IATOM,2))
  40    CONTINUE
	END




	SUBROUTINE MLTFORCE(FMLT)
************************************************************
*   Calculate multipole contribution to the electrostatic  *
*   force. Written by S.Yu.Savrasov                        *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER IS(NPAT),ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &    ,LMSYM(LMMAXV,NPAT)
	COMPLEX*16 VPS(0:NPLWMAX,NSPINMAX),RHOPS(0:NPLWMAX,NSPINMAX)
     &    ,AMTOT(NSYMMAX,NPAT,NSPINMAX),PSMOM(NSYMMAX,NPAT,NSPINMAX)
     &    ,QMAD(2,NSYMMAX,NPAT,NSPINMAX),RHOPS1(0:NPLWMAX,NSPINMAX)
     &    ,DQMAD(NSYMMAX,NPAT,NSPINMAX,3),YLM(LMMAXV),EXPT
     &    ,DPMOM(NSYMMAX,NPAT,NSPINMAX,3)
     &    ,PMOMT(NSYMMAX,NPAT,NSPINMAX),PSMOM1(NSYMMAX,NPAT,NSPINMAX)
     &    ,QMADT(NSYMMAX,NPAT,NSPINMAX)
	COMPLEX*16 FMLT(3,NPAT,4)
	DIMENSION SMT(NSORTMAX),Z(NSORTMAX),ZCOR(NSORTMAX)
     &    ,BET(0:LMAXV,NSORTMAX),GPLW(3,0:NPLWMAX),BES(-1:LMAXV+1)
     &    ,ORTH(2:3),TAU(3,NPAT),BESD(0:LMAXV+1)
	LOGICAL SYM(LMMAXV,NPAT)
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF            !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM    !!! from init
	COMMON /POTCOM/ SWZ,SMT                          !!! from vfull
	COMMON /BASIS/  TAU                              !!! from init
	COMMON /PSVCOM/ VPS                              !!! from vpseudo
	COMMON /PSRHO/  RHOPS,RHOPS1                     !!! from vcftr
	COMMON /LATPAR/ PAR,ORTH,OMEGAC                  !!! from init
	COMMON /BETPAR/ BET                              !!! from multro
	COMMON /MULTRO/ Z,ZCOR,AMTOT,PSMOM,PSMOM1,QMAD   !!! from multro
	COMMON /SYMMLM/ SYM,LMSYM                        !!! from symmlm 
	COMMON /PLWMESH/NPLW,GPLW                        !!! from plwmsh
	COMMON /SPIN/   NSPIN                            !!! from init
	COMMON /WORD/   NVAL                             !!! from nmtdata
	DATA PI/3.141592653589793D0/
	DO 10 IATOM=1,NATOM
	DO 10 IXYZ=1,3
	DO 10 IC=1,4
	FMLT(IXYZ,IATOM,IC)=(0.D0,0.D0)
  10	CONTINUE
	IF(NPF.EQ.0)RETURN
	SQPI4=SQRT(4.D0*PI)
C  CALCULATE CHANGES IN MULTIPOLE CHARGE DUE TO ATOMIC MOVEMENTS
	LMM=0
	ZPSD=0.D0
	DO 50 ISPIN=1,NSPIN
	ZPSD=ZPSD+OMEGAC*RHOPS(0,ISPIN)
	DO 40 IATOM=1,NATOM
	ISORT=IS(IATOM)
	LMM=MAX(LMM,ILSV(ISORT))
	DO 30 L=0,ILSV(ISORT)
	DO 20 M=-L,L
	LM=L*(L+1)+M+1
	DO 20 IXYZ=1,3
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	DQMAD(ISYM,IATOM,ISPIN,IXYZ)=(0.D0,0.D0)
	DPMOM(ISYM,IATOM,ISPIN,IXYZ)=(0.D0,0.D0)
	PMOMT(ISYM,IATOM,ISPIN     )=(0.D0,0.D0)
	QMADT(ISYM,IATOM,ISPIN     )=(0.D0,0.D0)
	ENDIF
  20    CONTINUE
  30    CONTINUE
  40    CONTINUE
  50    CONTINUE
	DO 140 IPLW=0,NPLW                   !!! over plane waves
	GX=GPLW(1,IPLW)*2.D0*PI/PAR
	GY=GPLW(2,IPLW)*2.D0*PI/PAR
	GZ=GPLW(3,IPLW)*2.D0*PI/PAR
	AG=SQRT(GX**2+GY**2+GZ**2)
	AG2=AG*AG
C  SET SPHERICAL HARMONICS
	IF(AG.GE.1.D-10)THEN
	CALL SPHHARM(GX/AG,GY/AG,GZ/AG,LMM,YLM)
	ELSE
	YLM(1)=1.D0/SQRT(4.D0*PI)
	DO 60 LM=2,(LMM+1)**2
	YLM(LM)=(0.D0,0.D0)
  60    CONTINUE
	ENDIF
C  SET CHANGE IN MULTIPOLE CHARGES
	IF(AG.GE.1.D-10)THEN                 !!! not for G=0 !
	DO 130 IATOM=1,NATOM                 !!! over atoms
	ISORT=IS(IATOM)
	CALL BESSR(AG*SMT(ISORT),BES(0),BESD(0),ILSV(ISORT)+1)
	BES(-1)=COS(AG*SMT(ISORT))/(AG*SMT(ISORT))
	SCP=GPLW(1,IPLW)*TAU(1,IATOM)+
     &      GPLW(2,IPLW)*TAU(2,IATOM)+
     &      GPLW(3,IPLW)*TAU(3,IATOM)
	EXPT=EXP((0.D0,-1.D0)*2.D0*PI*SCP)
	DO 120 ISPIN=1,NSPIN                 !!! over spins
	DO 110 L=0,ILSV(ISORT)
	DO 100 M=-L,L
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	QMADT(ISYM,IATOM,ISPIN)=QMADT(ISYM,IATOM,ISPIN)+
     &  4.D0*PI*SMT(ISORT)/DBLE(2*L+1)*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*4.D0*PI*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L-1)
	DQMAD(ISYM,IATOM,ISPIN,1)=DQMAD(ISYM,IATOM,ISPIN,1)+
     &  4.D0*PI*SMT(ISORT)/DBLE(2*L+1)*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*4.D0*PI*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L-1)*(0.D0,1.D0)*GX
	DQMAD(ISYM,IATOM,ISPIN,2)=DQMAD(ISYM,IATOM,ISPIN,2)+
     &  4.D0*PI*SMT(ISORT)/DBLE(2*L+1)*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*4.D0*PI*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L-1)*(0.D0,1.D0)*GY
	DQMAD(ISYM,IATOM,ISPIN,3)=DQMAD(ISYM,IATOM,ISPIN,3)+
     &  4.D0*PI*SMT(ISORT)/DBLE(2*L+1)*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*4.D0*PI*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L-1)*(0.D0,1.D0)*GZ
        PMOMT(ISYM,IATOM,ISPIN)=PMOMT(ISYM,IATOM,ISPIN)+
     &  SQPI4*4.D0*PI*SMT(ISORT)**2*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L+1)
        DPMOM(ISYM,IATOM,ISPIN,1)=DPMOM(ISYM,IATOM,ISPIN,1)+
     &  SQPI4*4.D0*PI*SMT(ISORT)**2*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L+1)*(0.D0,1.D0)*GX
        DPMOM(ISYM,IATOM,ISPIN,2)=DPMOM(ISYM,IATOM,ISPIN,2)+
     &  SQPI4*4.D0*PI*SMT(ISORT)**2*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L+1)*(0.D0,1.D0)*GY
        DPMOM(ISYM,IATOM,ISPIN,3)=DPMOM(ISYM,IATOM,ISPIN,3)+
     &  SQPI4*4.D0*PI*SMT(ISORT)**2*(RHOPS(IPLW,ISPIN)+
     &  RHOPS1(IPLW,ISPIN))*DCONJG(EXPT*YLM(LM))/AG*
     &  BES(L+1)*(0.D0,1.D0)*GZ
	ENDIF
 100    CONTINUE
 110    CONTINUE
 120    CONTINUE                             !!! over spins
 130    CONTINUE                             !!! over atoms
	ENDIF
 140    CONTINUE                             !!! over plane waves
C  SET MULTIPOLE CONTRIBUTION TO THE COULOMB FORCE
	DO 250 INUC=1,NATOM                  !!! over movements
	DO 240 IXYZ=1,3                      !!! over x,y,z
	FMLT(IXYZ,INUC,1)=(0.D0,0.D0)
	FMLT(IXYZ,INUC,2)=(0.D0,0.D0)
	FMLT(IXYZ,INUC,3)=(0.D0,0.D0)
	FMLT(IXYZ,INUC,4)=(0.D0,0.D0)
	IATOM=INUC                           !!! 
	ISORT=IS(IATOM)
	DO 230 ISPIN=1,NSPIN                 !!! over spins
	DO 220 L=0,ILSV(ISORT)               !!! over l
	DO 210 M=-L,L                        !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	FMLT(IXYZ,INUC,1)=FMLT(IXYZ,INUC,1)+1.D0/SQPI4*
     &  (DCONJG(PMOMT(ISYM,IATOM,ISPIN))*DQMAD(ISYM,IATOM,ISPIN,IXYZ))
	FMLT(IXYZ,INUC,1)=FMLT(IXYZ,INUC,1)-1.D0/SQPI4*
     &  (DCONJG(DPMOM(ISYM,IATOM,ISPIN,IXYZ))*QMADT(ISYM,IATOM,ISPIN))
	ENDIF
 210	CONTINUE                             !!! over m
 220	CONTINUE                             !!! over l
 230	CONTINUE                             !!! over spins
 240	CONTINUE                             !!! over x,y,z
 250	CONTINUE                             !!! over movements
	END


	SUBROUTINE MT0FORCE(FMT0)
*****************************************************************
*  Calculate MT-zero contribution to the total force.           *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &    ,IS(NPAT),NHSORT(NSORTMAX),LMSYM(LMMAXV,NPAT)
     &    ,NRAD(NSORTMAX)
	COMPLEX*16 V(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,RHOPS(0:NPLWMAX,NSPINMAX),VPS(0:NPLWMAX,NSPINMAX)
     &    ,RO(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,ROVALDS(3,NSYMMAX,NPAT,NSPINMAX)
     &    ,WORK2(0:NRADMAX),WORK1(0:NRADMAX)
     &    ,VPSINT(0:NPLWMAX,NSPINMAX)
        COMPLEX*16 FMT0(3,NPAT,4),DINT,FSRF(-1:1),CGNT
	DIMENSION H(NSORTMAX),R(0:NRADMAX,NSORTMAX),ORTH(2:3)
     &    ,WEIGHT(0:NRADMAX,NSORTMAX),DR(0:NRADMAX,NSORTMAX)
     &    ,SMT(NSORTMAX),RL(0:NRADMAX,0:LMAXV,NSORTMAX)
     &    ,VMT(NSORTMAX,NSPINMAX),VSF(NSORTMAX,NSPINMAX)
     &    ,VSPH(0:NRADMAX,NSORTMAX,NSPINMAX),GPLW(3,0:NPLWMAX)
     &    ,C(0:LMAXV+1,LMMAXV1,LMMAXV1),G2PLW(3,0:9*NPLWMAX)
     &    ,ROCOR(0:NRADMAX,NSORTMAX,NSPINMAX)
	REAL*8  WORK3(0:NRADMAX),WORK4(0:NRADMAX),TR1,TR2
	LOGICAL SYM(LMMAXV,NPAT)
	CHARACTER TXTEL(NSORTMAX)*10
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT!!! from init
	COMMON /POTCOM/ SWZ,SMT,VSPH,V                      !!! from vfull
	COMMON /PSVCOM/ VPS,VPSINT                          !!! from vfull
	COMMON /DENS/   RO,ROVALDS,ROCOR                    !!! from rofull
	COMMON /PSRHO/  RHOPS                               !!! from ttrint2
	COMMON /VZERO/  VMT0,VMT,VSF0,VSF                   !!! from vzero
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from symmlm
	COMMON /RADMESH/NRAD,H,R,DR,WEIGHT,RL               !!! from init
        COMMON /PLWMESH/NPLW,GPLW                           !!! from plwmsh
	COMMON /PLW2MSH/N2PLW,G2PLW                         !!! from plw2msh
	COMMON /GAUNT/  C                                   !!! from gdata
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /LATPAR/ PAR,ORTH,OMEGAC                     !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
	DO 10 IATOM=1,NATOM
	DO 10 IXYZ=1,3
	DO 10 IC=1,4
	FMT0(IXYZ,IATOM,IC)=(0.D0,0.D0)
  10	CONTINUE
	IF(NPF.NE.0)THEN
	DO 330 ISPIN=1,NSPIN                       !!! over spins
C  Calculate Fmt0=Int(grad(rho)*V0)
	DO 320 INUC=1,NATOM                        !!! over displacements
	DO 310 I2PLW=0,NPLW                        !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	DINT=DINT*RHOPS(I2PLW,ISPIN)*2.D0*PI/PAR*VMT0*(0.D0,1.D0)
	FMT0(1,INUC,1)=FMT0(1,INUC,1)+G2PLW(1,I2PLW)*DINT
	FMT0(2,INUC,1)=FMT0(2,INUC,1)+G2PLW(2,I2PLW)*DINT
	FMT0(3,INUC,1)=FMT0(3,INUC,1)+G2PLW(3,I2PLW)*DINT
 310	CONTINUE                                   !!! over palne waves 
 320	CONTINUE                                   !!! over displacements
 330	CONTINUE                                   !!! over spins
	ENDIF
C  ESTIMATE SURFACE INTEGRALS FROM CHARGE DENSITY AND POTENTIAL
	DO 580 INUC=1,NATOM                        !!! over inuc
	ISORT=IS(INUC)
	DO 570 MY=-1,1                             !!! over my
	FSRF(MY)=(0.D0,0.D0)
	DO 560 L1=0,ILSV(ISORT)                    !!! over l1
	DO 550 M1=-L1,L1                           !!! over m1
	L1M1=L1*(L1+1)+M1+1
	IF(.NOT.SYM(L1M1,INUC))GOTO550
	ISYM1=LMSYM(L1M1,INUC)
	DO 540 L=0,0                               !!! over l
	DO 530 M=-L,L                              !!! over m
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,INUC))GOTO530
	ISYM=LMSYM(LM,INUC)
        DO 520 L2=ABS(L1-L),L1+L,2                 !!! over l2
        M2=M-M1
        IF(L2.EQ.1.AND.M2.EQ.-MY)THEN
	CGNT=(0.D0,1.D0)**(L-L1-L2)*C(L2/2,LM,L1M1)
	DO 510 ISPIN=1,NSPIN                       !!! over spins
	FSRF(MY)=FSRF(MY)+(0.D0,1.D0)*SQRT(4.D0*PI/3.D0)*
     &  (-1)**MY*SMT(ISORT)**2*CGNT*
     &  DCONJG(RO(NRAD(ISORT),ISYM1,INUC,ISPIN))*VMT0*SQRT(4.D0*PI)
 510 	CONTINUE                                   !!! over spins
	ENDIF
 520    CONTINUE                                   !!! over l2
 530 	CONTINUE                                   !!! over m
 540 	CONTINUE                                   !!! over l
 550	CONTINUE                                   !!! over m1
 560	CONTINUE                                   !!! over l1
 570	CONTINUE                                   !!! over my
	CALL TRANSFER(FSRF(-1),FMT0(1,INUC,4))
 580	CONTINUE                                   !!! over inuc
	END



	SUBROUTINE PTNFORCE(FPTN)
*****************************************************************
*  Calculate potential contribution to the total force          *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &    ,IS(NPAT),NHSORT(NSORTMAX),LMSYM(LMMAXV,NPAT)
     &    ,NRAD(NSORTMAX)
	COMPLEX*16 V(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,RHOPS(0:NPLWMAX,NSPINMAX),VPS(0:NPLWMAX,NSPINMAX)
     &    ,RO(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,ROVALDS(3,NSYMMAX,NPAT,NSPINMAX)
     &    ,WORK2(0:NRADMAX),WORK1(0:NRADMAX)
     &    ,RHOPS1(0:NPLWMAX,NSPINMAX)
     &    ,VPSINT(0:NPLWMAX,NSPINMAX)
        COMPLEX*16 FPTN(3,NPAT,4),DINT,FSRF(-1:1),CGNT,FCOR(-1:1)
     &    ,PROD(0:9*NPLWMAX),TMPARR(0:NPLWMAX)
	DIMENSION H(NSORTMAX),R(0:NRADMAX,NSORTMAX),ORTH(2:3)
     &    ,WEIGHT(0:NRADMAX,NSORTMAX),DR(0:NRADMAX,NSORTMAX)
     &    ,SMT(NSORTMAX),RL(0:NRADMAX,0:LMAXV,NSORTMAX)
     &    ,VMT(NSORTMAX,NSPINMAX),VSF(NSORTMAX,NSPINMAX)
     &    ,VSPH(0:NRADMAX,NSORTMAX,NSPINMAX),GPLW(3,0:NPLWMAX)
     &    ,C(0:LMAXV+1,LMMAXV1,LMMAXV1),G2PLW(3,0:9*NPLWMAX)
     &    ,ROCOR(0:NRADMAX,NSORTMAX,NSPINMAX)
	REAL*8  WORK3(0:NRADMAX),WORK4(0:NRADMAX),TR1,TR2
	LOGICAL SYM(LMMAXV,NPAT)
	CHARACTER TXTEL(NSORTMAX)*10
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT!!! from init
	COMMON /POTCOM/ SWZ,SMT,VSPH,V                      !!! from vfull
	COMMON /PSVCOM/ VPS,VPSINT                          !!! from vfull
	COMMON /DENS/   RO,ROVALDS,ROCOR                    !!! from rofull
	COMMON /PSRHO/  RHOPS,RHOPS1                        !!! from ttrint2
	COMMON /VZERO/  VMT0,VMT,VSF0,VSF                   !!! from vzero
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from symmlm
	COMMON /RADMESH/NRAD,H,R,DR,WEIGHT,RL               !!! from init
        COMMON /PLWMESH/NPLW,GPLW                           !!! from plwmsh
	COMMON /PLW2MSH/N2PLW,G2PLW                         !!! from plw2msh
	COMMON /STORE1/ PROD,TMPARR                         !!! tmp.storage
	COMMON /GAUNT/  C                                   !!! from gdata
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /LATPAR/ PAR,ORTH,OMEGAC                     !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
	DO 10 IATOM=1,NATOM
	DO 10 IXYZ=1,3
	DO 10 IC=1,4
	FPTN(IXYZ,IATOM,IC)=(0.D0,0.D0)
  10	CONTINUE
	IF(NPF.EQ.1)THEN !!! slow calculation
	DO 220 ISPIN=1,NSPIN                       !!! over spins
	DO 210 IXYZ=1,3                            !!! over polarizations
C  Calculate Fptn1=Int[grad(rho)*Veff]
	DO 100 IPLW=0,NPLW                         !!! over plane waves
	TMPARR(IPLW)=RHOPS(IPLW,ISPIN)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 100	CONTINUE                                   !!! over plane waves
	CALL RHOVSUB(TMPARR,VPS(0,ISPIN),PROD)
	DO 120 INUC=1,NATOM                        !!! over displacements
	DO 110 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FPTN(IXYZ,INUC,1)=FPTN(IXYZ,INUC,1)+PROD(I2PLW)*DINT
 110	CONTINUE                                   !!! over palne waves 
 120	CONTINUE                                   !!! over displacements
C  Calculate Fptn2=Int[rho*grad(Veff)]
	DO 130 IPLW=0,NPLW                         !!! over plane waves
	TMPARR(IPLW)=VPS(IPLW,ISPIN)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 130	CONTINUE                                   !!! over plane waves
	CALL RHOVSUB(RHOPS(0,ISPIN),TMPARR,PROD)
	DO 150 INUC=1,NATOM                        !!! over displacements
	DO 140 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FPTN(IXYZ,INUC,2)=FPTN(IXYZ,INUC,2)+PROD(I2PLW)*DINT
 140 	CONTINUE                                   !!! over palne waves 
 150	CONTINUE                                   !!! over displacements
 210 	CONTINUE                                   !!! over polarizations
 220	CONTINUE                                   !!! over spins
	ELSEIF(NPF.EQ.2)THEN !!! fast calculation
	DO 330 ISPIN=1,NSPIN                       !!! over spins
C  Calculate Fpnt1=Int[grad(rho*Veff)]
	CALL RHOVSUB(RHOPS(0,ISPIN),VPS(0,ISPIN),PROD)
	DO 320 INUC=1,NATOM                        !!! over displacements
	DO 310 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	DINT=DINT*PROD(I2PLW)*(0.D0,1.D0)*2.D0*PI/PAR
	FPTN(1,INUC,1)=FPTN(1,INUC,1)+G2PLW(1,I2PLW)*DINT
	FPTN(2,INUC,1)=FPTN(2,INUC,1)+G2PLW(2,I2PLW)*DINT
	FPTN(3,INUC,1)=FPTN(3,INUC,1)+G2PLW(3,I2PLW)*DINT
 310	CONTINUE                                   !!! over plane waves 
 320	CONTINUE                                   !!! over displacements
 330	CONTINUE                                   !!! over spins
	ENDIF
C  ESTIMATE SURFACE INTEGRALS FROM CHARGE DENSITY AND POTENTIAL
	DO 580 INUC=1,NATOM                        !!! over inuc
	ISORT=IS(INUC)
	DO 570 MY=-1,1                             !!! over my
	FSRF(MY)=(0.D0,0.D0)
	FCOR(MY)=(0.D0,0.D0)
	DO 560 L1=0,ILSV(ISORT)                    !!! over l1
	DO 550 M1=-L1,L1                           !!! over m1
	L1M1=L1*(L1+1)+M1+1
	IF(.NOT.SYM(L1M1,INUC))GOTO550
	ISYM1=LMSYM(L1M1,INUC)
	DO 540 L=0,ILSV(ISORT)                     !!! over l
	DO 530 M=-L,L                              !!! over m
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,INUC))GOTO530
	ISYM=LMSYM(LM,INUC)
        DO 520 L2=ABS(L1-L),L1+L,2                 !!! over l2
        M2=M-M1
        IF(L2.EQ.1.AND.M2.EQ.-MY)THEN
	CGNT=(0.D0,1.D0)**(L-L1-L2)*C(L2/2,LM,L1M1)
	DO 510 ISPIN=1,NSPIN                       !!! over spins
	FSRF(MY)=FSRF(MY)+(0.D0,1.D0)*SQRT(4.D0*PI/3.D0)*
     &  (-1)**MY*SMT(ISORT)**2*CGNT*
     &  DCONJG(RO(NRAD(ISORT),ISYM1,INUC,ISPIN))*
     &  V(NRAD(ISORT),ISYM,INUC,ISPIN)
	IF(L1.EQ.0)
     &	FCOR(MY)=FCOR(MY)+(0.D0,1.D0)*SQRT(4.D0*PI/3.D0)*
     &  (-1)**MY*SMT(ISORT)**2*CGNT*
     &  ROCOR(NRAD(ISORT),ISORT,ISPIN)*
     &  V(NRAD(ISORT),ISYM,INUC,ISPIN)
 510 	CONTINUE                                   !!! over spins
	ENDIF
 520    CONTINUE                                   !!! over l2
 530 	CONTINUE                                   !!! over m
 540 	CONTINUE                                   !!! over l
 550	CONTINUE                                   !!! over m1
 560	CONTINUE                                   !!! over l1
 570	CONTINUE                                   !!! over my
	CALL TRANSFER(FCOR(-1),FPTN(1,INUC,3))
	CALL TRANSFER(FSRF(-1),FPTN(1,INUC,4))
 580	CONTINUE                                   !!! over inuc
	END




	SUBROUTINE COUFORCE(FCOU)
**************************************************************
*  Calculate electrostatic contribution to the total force   *
**************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &   ,IS(NPAT),NHSORT(NSORTMAX),LMSYM(LMMAXV,NPAT)
     &   ,NRAD(NSORTMAX)
	COMPLEX*16 V(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &   ,RHOPS(0:NPLWMAX,NSPINMAX),VPS(0:NPLWMAX,NSPINMAX)
     &   ,RO(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &   ,ROVALDS(3,NSYMMAX,NPAT,NSPINMAX)
     &   ,WORK1(0:NRADMAX),WORK2(0:NRADMAX)
     &   ,VPSINT(0:NPLWMAX,NSPINMAX)
     &   ,RHOPS1(0:NPLWMAX,NSPINMAX)
        COMPLEX*16 FCOU(3,NPAT,4),DINT,CGNT,FSRF(-1:1)
     &   ,PROD(0:9*NPLWMAX),TMPARR(0:NPLWMAX),RHOTMP(0:NPLWMAX)
	DIMENSION H(NSORTMAX),R(0:NRADMAX,NSORTMAX),ORTH(2:3)
     &   ,WEIGHT(0:NRADMAX,NSORTMAX),DR(0:NRADMAX,NSORTMAX)
     &   ,SMT(NSORTMAX),RL(0:NRADMAX,0:LMAXV,NSORTMAX)
     &   ,VSPH(0:NRADMAX,NSORTMAX,NSPINMAX),GPLW(3,0:NPLWMAX)
     &   ,C(0:LMAXV+1,LMMAXV1,LMMAXV1),G2PLW(3,0:9*NPLWMAX)
     &   ,ROCOR(0:NRADMAX,NSORTMAX,NSPINMAX)
     &   ,Z(NSORTMAX),ZCOR(NSORTMAX)
	REAL*8 WORK3(0:NRADMAX),WORK4(0:NRADMAX),TR1,TR2
	LOGICAL SYM(LMMAXV,NPAT)
	CHARACTER TXTEL(NSORTMAX)*10
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT!!! from init
	COMMON /POTCOM/ SWZ,SMT,VSPH,V                      !!! from vfull
	COMMON /DENS/   RO,ROVALDS,ROCOR                    !!! from rofull
	COMMON /PSVCOM/ VPS,VPSINT                          !!! from vfull
	COMMON /PSRHO/  RHOPS,RHOPS1                        !!! from ttrint2
	COMMON /MULTRO/ Z,ZCOR                              !!! from init
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from symmlm
	COMMON /RADMESH/NRAD,H,R,DR,WEIGHT,RL               !!! from init
        COMMON /PLWMESH/NPLW,GPLW                           !!! from plwmsh
	COMMON /PLW2MSH/N2PLW,G2PLW                         !!! from plw2msh
	COMMON /STORE1/ PROD,TMPARR,RHOTMP                  !!! tmp.storage
	COMMON /GAUNT/  C                                   !!! from gdata
	COMMON /LATPAR/ PAR,ORTH,OMEGAC                     !!! from init
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
	DO 10 IATOM=1,NATOM
	DO 10 IXYZ=1,3
	DO 10 IC=1,4
	FCOU(IXYZ,IATOM,IC)=(0.D0,0.D0)
  10	CONTINUE
C  SET SURFACE CONTRIBUTION TO THE COULOMB FORCE
	IF(NPF.EQ.1)THEN !!! slow calculation
	DO 220 ISPIN=1,NSPIN                       !!! over spins
	DO 210 IXYZ=1,3                            !!! over polarizations
	DO 100 IPLW=0,NPLW                         !!! over plane waves
	RHOTMP(IPLW)=RHOPS(IPLW,ISPIN)+RHOPS1(IPLW,ISPIN)
	TMPARR(IPLW)=RHOTMP(IPLW)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 100	CONTINUE                                   !!! over plane waves
C  Calculate Fcou1=Intr(grad(rho+rho1)VC]
	CALL RHOVSUB(TMPARR(0),VPS(0,ISPIN),PROD)
	DO 120 INUC=1,NATOM                        !!! over displacements
	DO 110 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FCOU(IXYZ,INUC,1)=FCOU(IXYZ,INUC,1)+PROD(I2PLW)*DINT
 110	CONTINUE                                   !!! over palne waves 
 120	CONTINUE                                   !!! over displacements
C  Calculate Fcou2=Int[(rho+rho1)grad(VC)]
	DO 125 IPLW=0,NPLW                         !!! over plane waves
	TMPARR(IPLW)=VPS(IPLW,ISPIN)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 125	CONTINUE                                   !!! over plane waves
	CALL RHOVSUB(RHOTMP(0),TMPARR(0),PROD)
	DO 140 INUC=1,NATOM                        !!! over displacements
	DO 130 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FCOU(IXYZ,INUC,2)=FCOU(IXYZ,INUC,2)+PROD(I2PLW)*DINT
 130 	CONTINUE                                   !!! over palne waves 
 140	CONTINUE                                   !!! over displacements
C  Calculate Fcou3=Int(grad(rho1)VC)
	DO 145 IPLW=0,NPLW                         !!! over plane waves
	TMPARR(IPLW)=RHOPS1(IPLW,ISPIN)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 145	CONTINUE                                   !!! over plane waves
	CALL RHOVSUB(TMPARR(0),VPS(0,ISPIN),PROD)
	DO 160 INUC=1,NATOM                        !!! over displacements
	DO 150 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FCOU(IXYZ,INUC,3)=FCOU(IXYZ,INUC,3)+PROD(I2PLW)*DINT
 150 	CONTINUE                                   !!! over palne waves 
 160	CONTINUE                                   !!! over displacements
 210 	CONTINUE                                   !!! over polarizations
 220	CONTINUE                                   !!! over spins
	ELSEIF(NPF.EQ.2)THEN !!! fast calculation
	DO 330 ISPIN=1,NSPIN                       !!! over spins
	DO 300 IPLW=0,NPLW                         !!! over plane waves
	RHOTMP(IPLW)=RHOPS(IPLW,ISPIN)+RHOPS1(IPLW,ISPIN)
 300	CONTINUE                                   !!! over plane waves
C  Calculate Fcou1=Int[grad[(rho+rho1)VC)]]
	CALL RHOVSUB(RHOTMP(0),VPS(0,ISPIN),PROD)
	DO 320 INUC=1,NATOM                        !!! over displacements
	DO 310 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	DINT=DINT*PROD(I2PLW)*(0.D0,1.D0)*2.D0*PI/PAR
	FCOU(1,INUC,1)=FCOU(1,INUC,1)+G2PLW(1,I2PLW)*DINT
	FCOU(2,INUC,1)=FCOU(2,INUC,1)+G2PLW(2,I2PLW)*DINT
	FCOU(3,INUC,1)=FCOU(3,INUC,1)+G2PLW(3,I2PLW)*DINT
 310	CONTINUE                                   !!! over palne waves 
 320	CONTINUE                                   !!! over displacements
 330	CONTINUE                                   !!! over spins
	ENDIF
C  ESTIMATE SURFACE INTEGRALS FROM CHARGE DENSITY AND POTENTIAL
	DO 580 INUC=1,NATOM                        !!! over inuc
	ISORT=IS(INUC)
	DO 570 MY=-1,1                             !!! over my
	FSRF(MY)=(0.D0,0.D0)
	DO 560 L1=0,ILSV(ISORT)                    !!! over l1
	DO 550 M1=-L1,L1                           !!! over m1
	L1M1=L1*(L1+1)+M1+1
	IF(.NOT.SYM(L1M1,INUC))GOTO550
	ISYM1=LMSYM(L1M1,INUC)
	DO 540 L=0,ILSV(ISORT)                     !!! over l
	DO 530 M=-L,L                              !!! over m
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,INUC))GOTO530
	ISYM=LMSYM(LM,INUC)
        DO 520 L2=ABS(L1-L),L1+L,2                 !!! over l2
        M2=M-M1
        IF(L2.EQ.1.AND.M2.EQ.-MY)THEN
	CGNT=(0.D0,1.D0)**(L-L1-L2)*C(L2/2,LM,L1M1)
	FSRF(MY)=FSRF(MY)+(0.D0,1.D0)*SQRT(4.D0*PI/3.D0)*
     &  (-1)**MY*SMT(ISORT)**2*CGNT*
     &  DCONJG(RO(NRAD(ISORT),ISYM1,INUC,1)+
     &  RO(NRAD(ISORT),ISYM1,INUC,NSPIN))/2.D0*DBLE(NSPIN)*
     &  (V(NRAD(ISORT),ISYM,INUC,1)+
     &  V(NRAD(ISORT),ISYM,INUC,NSPIN))/2.D0
	IF(L.EQ.0)FSRF(MY)=FSRF(MY)+(0.D0,1.D0)*SQRT(4.D0*PI/3.D0)*
     &  (-1)**MY*SMT(ISORT)**2*CGNT*
     &  DCONJG(RO(NRAD(ISORT),ISYM1,INUC,1)+
     &  RO(NRAD(ISORT),ISYM1,INUC,NSPIN))/2.D0*DBLE(NSPIN)*
     &  (-2.D0*Z(ISORT))/SMT(ISORT)*SQRT(4.D0*PI)
	ENDIF
 520    CONTINUE                                   !!! over l2
 530 	CONTINUE                                   !!! over m
 540 	CONTINUE                                   !!! over l
 550	CONTINUE                                   !!! over m1
 560	CONTINUE                                   !!! over l1
 570	CONTINUE                                   !!! over my
	CALL TRANSFER(FSRF(-1),FCOU(1,INUC,4))
 580	CONTINUE                                   !!! over inuc
	END


	SUBROUTINE EXCFORCE(FEXC)
*********************************************************
*  Calculate exchange-correlation contribution to the   *
*  total force.                                         *
*********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &   ,IS(NPAT),NHSORT(NSORTMAX),LMSYM(LMMAXV,NPAT)
     &   ,NRAD(NSORTMAX)
	COMPLEX*16 V(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &   ,RHOPS(0:NPLWMAX,NSPINMAX),VPS(0:NPLWMAX,NSPINMAX)
     &   ,RO(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &   ,WORK1(0:NRADMAX),WORK2(0:NRADMAX)
     &   ,ROVALDS(3,NSYMMAX,NPAT,NSPINMAX)
     &   ,VPSINT(0:NPLWMAX,NSPINMAX)
     &   ,RHOPS1(0:NPLWMAX,NSPINMAX)
        COMPLEX*16 FEXC(3,NPAT,4),DINT,FSRF(-1:1),CGNT
     &   ,PROD(0:9*NPLWMAX),TMPARR(0:NPLWMAX)
	DIMENSION H(NSORTMAX),R(0:NRADMAX,NSORTMAX),ORTH(2:3)
     &   ,WEIGHT(0:NRADMAX,NSORTMAX),DR(0:NRADMAX,NSORTMAX)
     &   ,SMT(NSORTMAX),RL(0:NRADMAX,0:LMAXV,NSORTMAX)
     &   ,VSPH(0:NRADMAX,NSORTMAX,NSPINMAX),GPLW(3,0:NPLWMAX)
     &   ,C(0:LMAXV+1,LMMAXV1,LMMAXV1),G2PLW(3,0:9*NPLWMAX)
     &   ,ROCOR(0:NRADMAX,NSORTMAX,NSPINMAX)
     &   ,Z(NSORTMAX),ZCOR(NSORTMAX)
	LOGICAL SYM(LMMAXV,NPAT)
	REAL*8 WORK3(0:NRADMAX),WORK4(0:NRADMAX),TR1,TR2
	CHARACTER TXTEL(NSORTMAX)*10
	COMMON /CONTROL/MAN,LMTO,NLR,NINT,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT!!! from init
	COMMON /POTCOM/ SWZ,SMT,VSPH,V                      !!! from vfull
	COMMON /PSVCOM/ VPS,VPSINT                          !!! from vfull
	COMMON /PSRHO/  RHOPS,RHOPS1                        !!! from ttrint2
	COMMON /DENS/   RO,ROVALDS,ROCOR                    !!! from rofull
	COMMON /RADMESH/NRAD,H,R,DR,WEIGHT,RL               !!! from init
        COMMON /PLWMESH/NPLW,GPLW                           !!! from plwmsh
	COMMON /PLW2MSH/N2PLW,G2PLW                         !!! from plw2msh
	COMMON /STORE1/ PROD,TMPARR                         !!! tmp.storage
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from symmlm
	COMMON /LATPAR/ PAR,ORTH,OMEGAC                     !!! from init
	COMMON /GAUNT/  C                                   !!! from gdata
	COMMON /EXCH/   IXC                                 !!! from init
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
	DO 10 IATOM=1,NATOM
	DO 10 IXYZ=1,3
	DO 10 IC=1,4
	FEXC(IXYZ,IATOM,IC)=(0.D0,0.D0)
  10	CONTINUE
C  SET EXCHANGE-CORRELATION FORCE
	IF(NPF.EQ.1)THEN !!! slow calculation
	DO 220 ISPIN=1,NSPIN                       !!! over spins
	DO 210 IXYZ=1,3                            !!! over polarizations
C  Calculate Fexc1=Int[grad(rho)*Vxc]
	DO 100 IPLW=0,NPLW                         !!! over plane waves
	TMPARR(IPLW)=RHOPS(IPLW,ISPIN)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 100	CONTINUE                                   !!! over plane waves
	CALL RHOVSUB(TMPARR,VPS(0,ISPIN),PROD)
	DO 120 INUC=1,NATOM                        !!! over displacements
	DO 110 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FEXC(IXYZ,INUC,1)=FEXC(IXYZ,INUC,1)+PROD(I2PLW)*DINT
 110	CONTINUE                                   !!! over palne waves 
 120	CONTINUE                                   !!! over displacements
C  Calculate Fexc2=Int[rho*grad(Vxc)]
	DO 125 IPLW=0,NPLW                         !!! over plane waves
	TMPARR(IPLW)=VPS(IPLW,ISPIN)*2.D0*PI/PAR*
     &  GPLW(IXYZ,IPLW)*(0.D0,1.D0)
 125	CONTINUE                                   !!! over plane waves
	CALL RHOVSUB(RHOPS(0,ISPIN),TMPARR(0),PROD)
	DO 140 INUC=1,NATOM                        !!! over displacements
	DO 130 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	FEXC(IXYZ,INUC,2)=FEXC(IXYZ,INUC,2)+PROD(I2PLW)*DINT
 130 	CONTINUE                                   !!! over palne waves 
 140	CONTINUE                                   !!! over displacements
 210 	CONTINUE                                   !!! over polarizations
 220	CONTINUE                                   !!! over spins
	ELSEIF(NPF.EQ.2)THEN !!! fast calculation
	DO 330 ISPIN=1,NSPIN                       !!! over spins
C  Calculate Fexc1=Int[grad(rho*Vxc)]
	CALL RHOVSUB(RHOPS(0,ISPIN),VPS(0,ISPIN),PROD)
	DO 320 INUC=1,NATOM                        !!! over displacements
	DO 310 I2PLW=0,N2PLW                       !!! over plane waves
	CALL DPLWMTS(G2PLW(1,I2PLW),G2PLW(2,I2PLW),G2PLW(3,I2PLW),
     &  DINT,INUC)
	DINT=DINT*PROD(I2PLW)*(0.D0,1.D0)*2.D0*PI/PAR
	FEXC(1,INUC,1)=FEXC(1,INUC,1)+G2PLW(1,I2PLW)*DINT
	FEXC(2,INUC,1)=FEXC(2,INUC,1)+G2PLW(2,I2PLW)*DINT
	FEXC(3,INUC,1)=FEXC(3,INUC,1)+G2PLW(3,I2PLW)*DINT
 310	CONTINUE                                   !!! over palne waves 
 320	CONTINUE                                   !!! over displacements
 330	CONTINUE                                   !!! over spins
	ENDIF
C  ESTIMATE SURFACE INTEGRALS FROM CHARGE DENSITY AND POTENTIAL
	DO 580 INUC=1,NATOM                        !!! over inuc
	ISORT=IS(INUC)
	DO 570 MY=-1,1                             !!! over my
	FSRF(MY)=(0.D0,0.D0)
	DO 560 L1=0,ILSV(ISORT)                    !!! over l1
	DO 550 M1=-L1,L1                           !!! over m1
	L1M1=L1*(L1+1)+M1+1
	IF(.NOT.SYM(L1M1,INUC))GOTO550
	ISYM1=LMSYM(L1M1,INUC)
	DO 540 L=0,ILSV(ISORT)                     !!! over l
	DO 530 M=-L,L                              !!! over m
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,INUC))GOTO530
	ISYM=LMSYM(LM,INUC)
        DO 520 L2=ABS(L1-L),L1+L,2                 !!! over l2
        M2=M-M1
        IF(L2.EQ.1.AND.M2.EQ.-MY)THEN
	CGNT=(0.D0,1.D0)**(L-L1-L2)*C(L2/2,LM,L1M1)
	DO 510 ISPIN=1,NSPIN                       !!! over spins
	FSRF(MY)=FSRF(MY)+(0.D0,1.D0)*SQRT(4.D0*PI/3.D0)*
     &  (-1)**MY*SMT(ISORT)**2*CGNT*
     &  DCONJG(RO(NRAD(ISORT),ISYM1,INUC,ISPIN))*
     &  V(NRAD(ISORT),ISYM,INUC,ISPIN)
 510 	CONTINUE                                   !!! over spins
	ENDIF
 520    CONTINUE                                   !!! over l2
 530 	CONTINUE                                   !!! over m
 540 	CONTINUE                                   !!! over l
 550	CONTINUE                                   !!! over m1
 560	CONTINUE                                   !!! over l1
 570	CONTINUE                                   !!! over my
	CALL TRANSFER(FSRF(-1),FEXC(1,INUC,4))
 580	CONTINUE                                   !!! over inuc
	END



	SUBROUTINE HLFFORCE(FHLF)
*****************************************************************
*  Calculate Hellmann-Feynman + Core Correction Contributions.  *
*****************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &    ,IS(NPAT),NHSORT(NSORTMAX),LMSYM(LMMAXV,NPAT)
     &    ,NRAD(NSORTMAX)
	COMPLEX*16 V(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX)
     &    ,AMTOT(NSYMMAX,NPAT,NSPINMAX),PSMOM(NSYMMAX,NPAT,NSPINMAX)
     &    ,WORK1(0:NRADMAX),WORK2(0:NRADMAX),DVDR(0:NRADMAX,NSYMMAX)
     &    ,RO(0:NRADMAX,NSYMMAX,NPAT,NSPINMAX),DVLMDR(0:NRADMAX)
     &    ,ROVALDS(3,NSYMMAX,NPAT,NSPINMAX),VR2(0:NRADMAX,NSYMMAX)
     &    ,PSMOM1(NSYMMAX,NPAT,NSPINMAX)
     &    ,QMAD(2,NSYMMAX,NPAT,NSPINMAX)
        COMPLEX*16 FHLF(3,NPAT,4),DINT,FSRF(-1:1),CGNT,FCOR(-1:1)
	DIMENSION H(NSORTMAX),R(0:NRADMAX,NSORTMAX),ORTH(2:3)
     &    ,WEIGHT(0:NRADMAX,NSORTMAX),DR(0:NRADMAX,NSORTMAX)
     &    ,SMT(NSORTMAX),RL(0:NRADMAX,0:LMAXV,NSORTMAX)
     &    ,VMT(NSORTMAX,NSPINMAX),VSF(NSORTMAX,NSPINMAX)
     &    ,VSPH(0:NRADMAX,NSORTMAX,NSPINMAX)
     &    ,ROCOR(0:NRADMAX,NSORTMAX,NSPINMAX)
     &    ,C(0:LMAXV+1,LMMAXV1,LMMAXV1)
     &    ,Z(NSORTMAX),ZCOR(NSORTMAX)
	LOGICAL SYM(LMMAXV,NPAT)
	CHARACTER TXTEL(NSORTMAX)*10
	COMMON /CONTROL/MAN,LMTO,NLR,NOVR,NPF               !!! from init
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT!!! from init
	COMMON /POTCOM/ SWZ,SMT,VSPH,V                      !!! from vfull
	COMMON /DENS/   RO,ROVALDS,ROCOR                    !!! from rofull
	COMMON /VZERO/  VMT0,VMT,VSF0,VSF                   !!! from vzero
	COMMON /SYMMLM/ SYM,LMSYM                           !!! from symmlm
	COMMON /STORE1/ VR2,DVDR                            !!! tmp.storage
	COMMON /RADMESH/NRAD,H,R,DR,WEIGHT,RL               !!! from init
        COMMON /MULTRO/ Z,ZCOR,AMTOT,PSMOM,PSMOM1,QMAD      !!! from multro
	COMMON /GAUNT/  C                                   !!! from gdata
	COMMON /SPIN/   NSPIN                               !!! from init
	COMMON /LATPAR/ PAR,ORTH,OMEGAC                     !!! from init
	COMMON /UNIT/   IUN                                 !!! from init
	COMMON /TXTEL/  TXTEL                               !!! from init
	DATA PI/3.141592653589793D0/
	SQPI4=2.D0*SQRT(PI)
	DO 1 IATOM=1,NATOM
	DO 1 IXYZ=1,3
	DO 1 IC=1,4
	FHLF(IXYZ,IATOM,IC)=(0.D0,0.D0)
   1	CONTINUE
	DO 40 INUC=1,NATOM                         !!! over displacements
	ISORT=IS(INUC)
	DO 30 MY=-1,1                              !!! over polarizations
	FSRF(MY)=(0.D0,0.D0)
	LM=3-MY
	IF(.NOT.SYM(LM,INUC))GOTO30
	ISYM=LMSYM(LM,INUC)
	CGNT=(0.D0,1.D0)/SQRT(3.D0)*(-1)**MY/SQRT(4.D0*PI)
	DO 20 ISPIN=1,NSPIN                        !!! over spins
	DO 10 IRAD=0,NRAD(ISORT)                   !!! over radial mesh
	WORK1(IRAD)=RO(IRAD,ISYM,INUC,ISPIN)*DR(IRAD,ISORT)
  10	CONTINUE                                   !!! over radial mesh
	CALL CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	FSRF(MY)=FSRF(MY)-Z(ISORT)*CGNT*(4.D0*PI*2.D0*
     &  WORK2(NRAD(ISORT))+3.D0*2.D0/SMT(ISORT)*(
     &  QMAD(1,ISYM,INUC,ISPIN)+QMAD(2,ISYM,INUC,ISPIN)))
  20	CONTINUE                                   !!! over spins
  30	CONTINUE                                   !!! over polarizations
	CALL TRANSFER(FSRF(-1),FHLF(1,INUC,1))
  40	CONTINUE                                   !!! over displacements
C  CALCULATE CORE CORRECTION AND Int(rho*grad(Veff))
	DO 250 INUC=1,NATOM                        !!! over displacements
	ISORT=IS(INUC)
	DO 240 MY=-1,1                             !!! over polarizations
	FSRF(MY)=(0.D0,0.D0)
	FCOR(MY)=(0.D0,0.D0)
	DO 230 ISPIN=1,NSPIN                       !!! over spins
	DO 120 L=0,ILSV(ISORT)                     !!! over l
	DO 110 M=-L,L                              !!! over m
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,INUC))GOTO110
	ISYM=LMSYM(LM,INUC)
	IF(ISYM.NE.1)THEN
	DO 50 IRAD=0,NRAD(ISORT)
	WORK1(IRAD)=V(IRAD,ISYM,INUC,ISPIN)
  50	CONTINUE
     	CALL DRSUB(WORK1,WORK2,NRAD(ISORT),H(ISORT),
     &  R(0,ISORT),DR(0,ISORT))
	DO 60 IRAD=0,NRAD(ISORT)
	DVDR(IRAD,ISYM)=WORK2(IRAD)*R(IRAD,ISORT)**2
	VR2(IRAD,ISYM)=V(IRAD,ISYM,INUC,ISPIN)*R(IRAD,ISORT)
  60	CONTINUE
	ELSEIF(ISYM.EQ.1)THEN
	WORK1(0)=2.D0*Z(ISORT)
	DO 70 IRAD=1,NRAD(ISORT)
	WORK1(IRAD)=-R(IRAD,ISORT)*V(IRAD,1,INUC,ISPIN)/
     &  SQPI4
  70	CONTINUE
     	CALL DRSUB(WORK1,WORK2,NRAD(ISORT),H(ISORT),
     &  R(0,ISORT),DR(0,ISORT))
	DVDR(0,ISYM)=2.D0*Z(ISORT)*SQPI4
	VR2(0,ISYM)=-2.D0*Z(ISORT)*SQPI4
	DO 80 IRAD=1,NRAD(ISORT)
	DVDR(IRAD,ISYM)=-R(IRAD,ISORT)*V(IRAD,1,INUC,ISPIN)-
     &  WORK2(IRAD)*R(IRAD,ISORT)*SQPI4
	VR2(IRAD,ISYM)=R(IRAD,ISORT)*V(IRAD,1,INUC,ISPIN)
  80	CONTINUE
	ENDIF
 110	CONTINUE                                   !!! over m
 120	CONTINUE                                   !!! over l
	DO 220 L=0,ILSV(ISORT)                     !!! over l
	DO 210 M=-L,L                              !!! over m
	LM=L*(L+1)+M+1
	IF(.NOT.SYM(LM,INUC))GOTO210
	ISYM=LMSYM(LM,INUC)
	LMMY=(L-1)*L+M-MY+1
	LMPY=(L+1)*(L+2)+M-MY+1
	DO 130 IRAD=0,NRAD(ISORT)                  !!! over radial mesh
	DVLMDR(IRAD)=(0.D0,0.D0)
	IF(L.GT.0.AND.ABS(M-MY).LE.L-1)THEN
	IF(SYM(LMMY,INUC))THEN
	ISYMMY=LMSYM(LMMY,INUC)
	CGNT=(0.D0,-1.D0)*SQRT(4.D0*PI/3.D0)*C(0,LM,LMMY)
	DVLMDR(IRAD)=DVLMDR(IRAD)+CGNT*(DVDR(IRAD,ISYMMY)-
     &  DBLE(L-1)*VR2(IRAD,ISYMMY))
	ENDIF
	ENDIF
	IF(L.LT.ILSV(ISORT))THEN
	IF(SYM(LMPY,INUC))THEN
	ISYMPY=LMSYM(LMPY,INUC)
	CGNT=(0.D0,+1.D0)*SQRT(4.D0*PI/3.D0)*C(0,LM,LMPY)
	DVLMDR(IRAD)=DVLMDR(IRAD)+CGNT*(DVDR(IRAD,ISYMPY)+
     &  DBLE(L+2)*VR2(IRAD,ISYMPY))
	ENDIF
	ENDIF
 130	CONTINUE                                   !!! over radial mesh
	DO 140 IRAD=0,NRAD(ISORT)                  !!! over radial mesh
	WORK1(IRAD)=DCONJG(RO(IRAD,ISYM,INUC,ISPIN))*DVLMDR(IRAD)*
     &  DR(IRAD,ISORT)
 140 	CONTINUE                                   !!! over radial mesh
	CALL CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	FSRF(MY)=FSRF(MY)+WORK2(NRAD(ISORT))
	IF(L.EQ.0)THEN
	DO 150 IRAD=0,NRAD(ISORT)                  !!! over radial mesh
	WORK1(IRAD)=ROCOR(IRAD,ISORT,ISPIN)*DVLMDR(IRAD)*
     &  DR(IRAD,ISORT)
 150	CONTINUE                                   !!! over radial mesh
	CALL CQD(H(ISORT),WORK1,WORK2,NRAD(ISORT)+1)
	FCOR(MY)=FCOR(MY)+WORK2(NRAD(ISORT))
	ENDIF
 210   	CONTINUE                                   !!! over m
 220   	CONTINUE                                   !!! over l
 230   	CONTINUE                                   !!! over spins
 240   	CONTINUE                                   !!! over polarizations
	CALL TRANSFER(FSRF(-1),FHLF(1,INUC,2))
	CALL TRANSFER(FCOR(-1),FHLF(1,INUC,3))
 250   	CONTINUE                                   !!! over displacements
	END



	SUBROUTINE TRANSFER(A,B)
************************************************************
*  Transfere from cyclic to cartesian system.              *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 TR(-1:1,3),A(-1:1),B(3)
        DATA TR/(+0.70710678118655D0, 0.00000000000000D0)
     &         ,( 0.00000000000000D0, 0.00000000000000D0)
     &         ,(-0.70710678118655D0, 0.00000000000000D0)
     &         ,( 0.00000000000000D0,+0.70710678118655D0)
     &         ,( 0.00000000000000D0, 0.00000000000000D0)
     &         ,( 0.00000000000000D0,+0.70710678118655D0)
     &         ,( 0.00000000000000D0, 0.00000000000000D0)
     &         ,( 1.00000000000000D0, 0.00000000000000D0)
     &         ,( 0.00000000000000D0, 0.00000000000000D0)/
	DO 20 I=1,3
	B(I)=(0.D0,0.D0)
	DO 10 MY=-1,1
	B(I)=B(I)+A(MY)*TR(MY,I)
  10    CONTINUE
  20    CONTINUE        
	END



        SUBROUTINE RHOVSUB(RHOPS,VPS,PROD)
*****************************************************************
*  Calculates product of rho(G') and v(G) as an expansion       *
*  in plane waves.                                              *
*****************************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        INCLUDE 'PARAM.DAT'
        INTEGER KVEC(3,0:NPLWMAX),KGMIN(3),KGMAX(3)
     &   ,INDG(NFFTMAX),INDG2(9*NFFTMAX)
        DIMENSION ORTH(2:3),GPLW(3,0:NPLWMAX),G2PLW(3,0:9*NPLWMAX)
        COMPLEX*16 RHOPS(0:NPLWMAX),VPS(0:NPLWMAX),PROD(0:9*NPLWMAX)
        COMMON /PLWMESH/NPLW,GPLW,KVEC,KGMIN,KGMAX,INDG       !!! from plwmsh
        COMMON /PLW2MSH/N2PLW,G2PLW,INDG2                     !!! from plwint
        COMMON /LATPAR/ PAR,ORTH,OMEGAC                       !!! from init
        DATA PI/3.141592653589793D0/
C  SET ZERO IN PROD-ARRAY
	DO 10 I2PLW=0,N2PLW
	PROD(I2PLW)=(0.D0,0.D0)
  10	CONTINUE
C  DO OVER PLANE WAVES
        M1D=2*KGMAX(1)-2*KGMIN(1)+1
        M2D=2*KGMAX(2)-2*KGMIN(2)+1
        M3D=2*KGMAX(3)-2*KGMIN(3)+1
        M12D=M1D*M2D
        M123D=M1D*M2D*M3D
        DO 30 IPLW=0,NPLW
        M1I=KVEC(1,IPLW)+2*KGMIN(1)
        M2I=KVEC(2,IPLW)+2*KGMIN(2)
        M3I=KVEC(3,IPLW)+2*KGMIN(3)
        DO 20 JPLW=0,NPLW
        M1=KVEC(1,JPLW)-M1I
        M2=KVEC(2,JPLW)-M2I
        M3=KVEC(3,JPLW)-M3I
        IDIM=1+M1+M1D*M2+M12D*M3
        I2PLW=INDG2(IDIM)
	PROD(I2PLW)=PROD(I2PLW)+DCONJG(RHOPS(IPLW))*VPS(JPLW)
  20	CONTINUE
  30	CONTINUE
	END
