	SUBROUTINE GDATA(LMM)
*************************************************************
*   Calculate the Gaunt coefficients.                       *
*   Calculate g-coefficients defined as follows:            *
*                                                           *
*                    -8pi(2l2-1)!!      l2m2                *
*    G(l2,l1m1,lm)=------------------- C      /sqrt(4pi)    *
*                   (2l1-1)!!(2l-1)!!   lm,l1m1             *
*   where C are the Gaunt coefficients                      *
*   Written by S.Yu.Savrasov (P.N.Lebedev Physical Institute)
*************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	DIMENSION C(0:LMAXV+1,LMMAXV1,LMMAXV1)
     &    ,GGNT(0:LMAXV+1,0:LMAXV+1,0:LMAXV+1)
	COMMON /GAUNT/C,GGNT
	DATA PI/3.141592653589793D0/
	CALL SGAUNT(C,LMM)     !!! calculate Gaunt coefficients
	DO 30 L1=0,LMM        !!! calculate g-coefficients
	DO 20 L =0,LMM
	 DO 10 L2=ABS(L1-L),L1+L,2
	 LL2=L2/2
	 G=-8.D0*PI/SQRT(4.D0*PI)*DFACT(2*L2-1)/DFACT(2*L1-1)/
     &   DFACT(2*L-1)
	 GGNT(LL2,L1,L)=G
  10     CONTINUE
  20    CONTINUE
  30    CONTINUE
	END

C******************************************************
	FUNCTION DFACT(N) !!! N MUST BE ODD NUMBER  !!!
C******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	IF(N.LT.-1)PRINT*,'WRONG DFACTORIAL'
	DFACT=1.0
	DO 10 I=1,N,2
	DFACT=DFACT*I
  10    CONTINUE
	END



	SUBROUTINE SGAUNT(C,LMM)
************************************************************
*   Calculation of the Gaunt coefficients C(L2M2,L1M1,LM)  *
*                                                          *
*    l2m2                    /         *                   *
*   C       =C(l2m2,l1m1,lm)=\dr*Y(r)*Y(r)*Y(r)            *
*    lm,l1m1                 /    lm   l1m1 l2m2           *
*                                                          *
*    and C.ne.0 when l2=/l1-l/,/l1-l/+2,...,l1+l,m2=m1-m   *
*    Y(lm) is a complex spherical garmonic with a phase    *
*    after Condon and Shortley                             *
* Written by S.Yu.Savrasov (P.N.Lebedev Physical Institute)*
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	DIMENSION C(0:LMAXV+1,LMMAXV1,LMMAXV1)
	DATA PI/3.141592653589793D0/
	DO 10 L1=0,LMM
	DO 10 M1=-L1,L1
	DO 10 L =0,LMM
	DO 10 M =-L,L
	 L1M1=L1*(L1+1)+M1+1
	 LM=L*(L+1)+M+1
	 DO 10 L2=ABS(L1-L),L1+L,2
	  LL2=L2/2
	  M2=M1-M
	  IF(ABS(M2).LE.L2)THEN     !!! selection rule
	  AJ=L
	  BJ=L2
	  AM=M
	  BM=M2
	  CJ=L1
	  CM=M1
	  A1=CLEBSCH(AJ,BJ,0.D0,0.D0,CJ,0.D0)  !!! Clebsch-Gordan coefficients
	  A2=CLEBSCH(AJ,BJ,AM,BM,CJ,CM)        !!! Clebsch-Gordan coefficients
	  DL1=2*L +1
	  DL2=2*L2+1
	  DL3=2*L1+1
	  C(LL2,L1M1,LM)=A1*A2*SQRT(DL1*DL2/DL3/4.D0/PI)
	  ELSE IF(ABS(M2).GT.L2)THEN
	  C(LL2,L1M1,LM)=0.0
	  ENDIF
  10    CONTINUE
	RETURN
	END




      FUNCTION CLEBSCH(AJ,BJ,AM,BM,CJ,CM)
******************************************************************
* Program calculates Clebsch-Gordan coefficients                 *
* See: Landau and Lifshitz, Vol.3                                *
*      cj,cm                                                     *
*     C                                                          *
*      aj,am,bj,bm                                               *
* Written by A.Soldatov (IAE)                                    *
******************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION F(100)
      SAVE F,N,K,X
      DATA N/100/, K/0/,  X/2.D0/
      F(1)=0.D0
      F(2)=0.D0
      IF(K)1,1,2
    1 K=1
      DO 3 I=3,N
      F(I)=F(I-1)+DLOG(X)
    3 X=X+1.D0
    2 I=AM+BM-CM+.1D0
      IF(I)100,4,100
    4 I1=AJ+BJ-CJ+1.1D0
      IF(I1)100,100,6
    6 I2=AJ-BJ+CJ+1.1D0
      IF(I2)100,100,7
    7 I3=BJ+CJ-AJ+1.1D0
      IF(I3)100,100,8
    8 X=AJ+BJ+CJ+2.1D0
      I4=X
      I=X+.6D0
      I=I4-I
      IF(I)100,5,100
    5 X=AJ+AM+1.1D0
      I5=X
      IF(I5)100,100,9
    9 I=X+.6D0
      I=I-I5
      IF(I)100,10,100
   10 I6=AJ-AM+1.1D0
      IF(I6)100,100,11
   11 X=BJ+BM+1.1D0
      I7=X
      IF(I7)100,100,12
   12 I=X+.6D0
      I=I-I7
      IF(I)100,13,100
   13 I8=BJ-BM+1.1D0
      IF(I8)100,100,14
   14 X=CJ+CM+1.1D0
      I9=X
      IF(I9)100,100,15
   15 I=X+.6D0
      I=I-I9
      IF(I)100,16,100
   16 I10=CJ-CM+1.1D0
      IF(I10)100,100,17
   17 X=F(I1)+F(I2)+F(I3)-F(I4)
      I=I5-I6
      IF(I)18,19,18
   19 I=I7-I8
      IF(I)18,200,18
   18 X=X+F(I5)+F(I6)+F(I7)+F(I8)+F(I9)+F(I10)
      X=X*.5D0
      I10=MIN0(I1,I6,I7)
      I2=I1-I5
      I3=I1-I8
      I9=MAX0(0,I2,I3)+1
      I1=I1+1
      I6=I6+1
      I7=I7+1
      I8=I9/2
      E=1.D0
      I5=I9*.5D0+.6D0
      I8=I8-I5
      IF(I8)20,21,20
   21 E=-1.D0
   20 S=0.D0
      DO 22 I=I9,I10
      C=X-F(I)-F(I1-I)-F(I6-I)-F(I7-I)-
     *F(I-I2)-F(I-I3)
      S=S+E*DEXP(C)
   22 E=1.D0-E-1.D0
      CLEBSCH=DSQRT(CJ+CJ+1.D0)*S
      RETURN
  200 I=I4/2
      I5=I4*.5D0+.6D0
      I=I-I5
      IF(I)100,201,100
  201 I6=I5-I6+1
      I7=I5-I8+1
      I8=I5-I10+1
      S=X*0.5D0+F(I5)-F(I6)-F(I7)-F(I8)
      S=DEXP(S)
      I5=I8/2
      I6=I8*.5D0+.6D0
      I5=I5-I6
      IF(I5)202,203,202
  203 S=1.D0-S-1.D0
  202 CLEBSCH=S*DSQRT(CJ+CJ+1.D0)
      RETURN
  100 CLEBSCH=0.D0
      RETURN
      END
